#!/usr/bin/perl

require 'lib.pl';

if(scalar(@ARGV) < 2)
{
  print <<EOF;

Index Generator
by Jonah Cohen <jonah\@ibiblio.org>
http://www.ibiblio.org/obp/ltoh/

Usage:
  $0 source_dir dest_dir [language]

EOF
  exit(0);
}

load($ARGV[2]);

open(TOP, "$ARGV[0]/top.tex") || die("Can't open $ARGV[0]/top.tex\n");
$_ = <TOP>;
close(TOP);
$previous = $1 while s/^[^%\n]*\\include\s*\{([^\}]*)\}//m;

open(INDEX, ".index_lst") || die("Can't open .index_lst\n");;
foreach (split /\n/,<INDEX>)
{
  /(.*)\t(.*)/;
  $index{$1} = $2;
}
close(INDEX);

open(OUTPUT, ">$ARGV[1]/dex.htm") || die("Can't write to $ARGV[1]/dex.htm\n");

open(HEADER, "html_header.html") || die("Can't read from html_header.html\n");
$header = languagestrings(<HEADER>);
close(HEADER);

open(HEADERFOOTER, "html_header_footer.html") || die("Can't read from html_header_footer.html\n");
$headerfooter = languagestrings(<HEADERFOOTER>);
close(HEADERFOOTER);

$header =~ s/\$title/$Index/g;

$headerfooter =~ s/\$next/index/;
$headerfooter =~ s/\$previous/$previous/;

print "Creating Index\n";

#write HTML header
print OUTPUT <<EOF;
$header
$headerfooter

<h1>$Index</h1>

<table border="0" cellspacing="0" cellpadding="20">
<tr>
<td valign="top" width="50%">
<font size=2>
EOF

#sort items alphabetically (case-insensitive)
@items = sort {letters($a) cmp letters($b)} keys %index;

sub letters
{
  $_ = uc pop;
  s/<[^>]+>//g;
  return $_;
}

$i = 0;
$n = int((scalar(@items)+1)/2);
foreach (@items)
{
  $i++;

  if(s/\|([^\{]*)\{\s*([^\}]*)\}?/... $1 $2/)
  {
  	($_, $b) = latex($_);
    print OUTPUT "$_<br>\n";
    next;
  }

  @links = split /,/, $index{$_};

  s/^.+\!(.+)$/&nbsp; &nbsp; &nbsp; $1/;
  s/\!//g;
  print OUTPUT "$_... ";

  while ($_ = shift @links)
  {
    /-/;
    $link = $';
    ($_, $b) = latex($`);
    print OUTPUT "<a href=$link>$_</a>";
    if(scalar(@links) > 0)
    {
      print OUTPUT ", ";
    }
    else
    {
      print OUTPUT "\n";
    }
  }

  if($i == $n)
  {
    print OUTPUT <<EOF
</font>
</td>
<td valign="top" width="50%">
<font size=2>
EOF
  }
  else
  {
    print OUTPUT "<br>";
  }
}

#write HTML footer
print OUTPUT <<EOF;
</font>
</tr>
</table>
$headerfooter
</body>
</html>
EOF

close(OUTPUT);
