undef $/;

$sec = 0;

sub latex
{
  my ($m, $p);

  $p = "";

  $_ = ignore($_[0]);

  while(/[\\\{\}\%\$\&\<\>\|]|([\"\'\`\n])\1|\-{3}|\w*~/)
  {
    if($` ne "")
    {
      $_ = $& . $';
      $p .= languagestrings($`);
    }
    else
    {
      return ("$_[2]$p$_[3]", $') if /^\}/ || /^$/ || $_[1] =~ /./ && /^$_[1]/;

      ($m, $_) = latextag($_, $_[1]);
      $p .= $m;
    }

    $_ = ignore($_);
  }

  return ("$_[2]$p$_$_[3]", "");
}

sub ignore
{
  $_ = $_[0];

  my ($m, $p);

  $p = "";

  while(1)
  {
    if(/^\n+/)
    {
      $_ = $';
      $p .= $&;
      next;
    }
    next if !(/^%!--feedback\s/) && s/^%[^\n]*//;
    next if s/^\\(pagebreak|noindent|before\S*|after\S*)//;
    next if s/^\\(begin|end)\s*\{flushleft\}//;
    if(/^\\(index|label|vspace|adjustpage)\s*\{/)
    {
      ($m, $_) = latex($', $_[1]);
      next;
    }
    if(/^\\(begin|end)\s*\{([^\}]+)\}/)
    {
      $m = $';

      unless($2 =~ /^(itemize|enumerate|description|quote|eqnarray|equation|verbatim|table|tabular)\*?$/)
      {
        $_ = $m;
        next;
      }
    }
    last;
  }

  return $p . $_;
}

sub latextag
{
  $_ = $_[0];

  my ($l, $m, $c);

  #feedback comments
  return ("<!--feedback $1 -->\n", $') if /^%!--feedback\s+([^\n]*)/;

  #new paragraph
  if(/^\n{2,}/)
  {
    $_ = ignore($');
    return (/^\\\w/ ? "\n\n" : "\n<p>\n", $_);
  }

  #special characters
  return ("<br>", $') if /^\\\\/;
  return ("&lt;", $') if /^\\?</;
  return ("&gt;", $') if /^\\?>/;
  return ("\"", $') if /^(``|'')/;
  return ($1, $') if /^\\([\_\#\&\%\{\}\[\]\$])/;
  return ("&copy;", $') if /^\\copyright/;
  return ("", $') if /^\\-/;
  return (" <s>&nbsp; &nbsp;</s> ", $') if /^---/;

  #accents
  my %accents = ("'","acute","`","grave","\"","uml","~","tilde","^","circ");
  return ("&$2$accents{$1};", $') if /^\\(['`"~^])\{\\?(\w)\}/;

  #references
  return ("<a href=\"$labellinks{$2}\">$1 $labels{$2}</a>", $') if /^(\w*)~\\ref\s*\{([^\}]*)\}/;

  #to ignore \Latex~input
  return ("$1 ", $') if /^\\(\w+)~/;

  #tt tags
  if(/^\{\\tt\s+/)
  {
    $_ = $';
    $l = "";

    #check for hyperlinks
    $l = "<a href=\"mailto:$1\">$1<\/a>" if /^(\S+@\S+)\s*\}/;
    $l = "<a href=\"$1\">$1<\/a>" if /^((ht|f)tp:\/\/\S+)\s*\}/;
    $l = "<a href=\"http:\/\/$1\">$1<\/a>" if /^(www\.\S+)\s*\}/;

    return ($l, $') if $l ne "" && /\}/;

    #no hyperlinks
    return latex($_, $_[1], "<code>", "</code>");
  }

  #basic formatting tags
  return latex($', $_[1], "<b>", "</b>") if /^\{\\bf\s*/;
  return latex($', $_[1], "<b>", "</b>") if /^\\textbf\s*\{/;
  return latex($', $_[1], "<i>", "</i>") if /^\{\\em\s*/;
  return ("<code>$2</code>", $') if /^\\verb(\W)(.*?)\1/;
  return ("<code>$2</code>", $') if /^\{\\verb(\W)(.*?)\1\}/;

  #part of index tags
  if(/^\|([^\{]*)\{/)
  {
    $m = $1;
    ($l, $_) = latex($', $_[1]);
    return ("|$m\{$l\}", $_);
  }

  return latex($& . $', $_[1]) if /^\|see\s*\{/;

  #footnotes
  if(/^\\footnote\s*\{/)
  {
    ($l, $_) = latex($', $_[1]);
    $l =~ s/\'/\\\'/g;
    return (" <a href=\"javascript:fn(\'$l\')\">* Note<\/a>", $_);
  }

  #chapters
  return latex($', $_[1], "<h1>", "</h1>") if /^\\chapter\s*\{/;

  #sections
  return latex($', $_[1], "\n<h3>", "</h3>") if /^\\section\*\s*\{/;
  if(/^\\section\s*\{/)
  {
    ($l, $_) = latex($', $_[1]);

    if($ischapter)
    {
      $sec++;
      return latex($', $_[1], "<a name=$sec><\/a>\n<h3>$chap.$sec ", "<\/h3>");
    }

    return latex($', $_[1], "\n<h3>", "<\/h3>");
  }

  #subsections and subsubsections
  return latex($', $_[1], "<h4>", "</h4>") if /^\\(sub)+section\*?\s*\{/;

  #list items
  return latex($', "\\n{2,}", "<dt>" . (join "",latex($1)) . "</dt>\n<dd>", "</dd>\n") if /^\\item\s*\[([^:\]]*):?\]\s*/;
  return latex($', "\\n{2,}", "<li>", "</li>\n") if /^\\item\s*/;

  #unordered lists
  return ("<ul>", $') if /^\\begin\s*\{itemize\}\n/;
  return ("</ul>", $') if /^\\end\s*\{itemize\}/;

  #ordered lists
  return ("<ol>", $') if /^\\begin\s*\{enumerate\}\n/;
  return ("</ol>", $') if /^\\end\s*\{enumerate\}/;

  #definition lists
  return ("<dl>", $') if /^\\begin\s*\{description\}\n/;
  return ("</dl>", $') if /^\\end\s*\{description\}/;

  #blockquotes
  return ("<blockquote>", $') if /^\\begin\s*\{quote\}\n/;
  return ("</blockquote>", $') if /^\\end\s*\{quote\}/;

  #images
  return ("<p align=\"center\"><img src=\"$1.png\"><\/p>", $') if /^\\centerline\s*\{\\psfig\s*\{figure=([^\.]*)[^\}]*\}\s*\}/;

  #centered text (?)
  return latex($', $_[1], "<b>", "</b>") if /^\\centerline\s*\{/;

  #math
  return math($', "<tt>", "</tt>") if /^\$/;
  return math($', "<p>\n<table align=center><tr><td><tt>\n", "</tt></td></tr></table>\n\n") if /^\\begin\s*\{(eqnarray|equation)\*?\}\n/;

  #verbatim
  return verbatim($', "<pre>", "</pre>") if /^\\begin\s*\{verbatim\*?\}\n/;

  #table
  return table($', "<p>\n<table align=center>", "</table>") if /^\\begin\s*\{(table|tabular)\*?\}/;


  #not recognized, so skip char
  return ($&, $') if /^./;
}

sub math
{
  my ($m, $p);

  $_ = $_[0];
  s/^\s+//;

  $p = "";

  while(/[\\\{\}\$\_\^\&]/)
  {
    $m = $& . $';
    $p .= $`;

    return ("$_[1]$p$_[2]", $') if $m =~ /^(\}|\$|\\end\s*\{(eqnarray|equation)\*?\})/;

    ($m, $_) = mathtag($m);

    $p .= $m;
  }

  return ("$_[1]$p$_$_[2]", "");
}

sub mathtag
{
  my ($l, $m);

  $_ = $_[0];

  return ("<br>", $') if /^\\\\\s*/;

  return ("", $') if /^&&\s*/;
  return ("=", $') if /^&\s+=\s+&/;
  return ("&copy;", $') if /^\\copyright/;

  #superscript and subscript
  return math($', "<sub>", "</sub>") if /^_\s*\{/;
  return ("<sub>$1</sub>", $') if /^_(.)/;
  return math($', "<sup>", "</sup>") if /^\^\s*\{/;
  return ("<sup>$1</sup>", $') if /^\^(.)/;

  #fractions
  if(/^\\frac\s*\{/)
  {
    ($l, $_) = math($', "<tr><td>", "<\/td><\/tr>");
    ($m, $_) = math($', "<tr><td><hr noshade color=black><\/td><\/tr><tr><td>", "<\/td><\/tr>") if /^\s*\{/;
    return ("<table style=\"display: inline; align: center; vertical-align: middle;\">$l$m</table>", $_);
  }

  #math functions
  return math($', "<table style=\"display: inline; vertical-align: middle;\"><tr><td rowspan=2 valign=bottom><font size=5>&radic;<\/font><\/td><td><hr noshade color=black><\/td><\/tr><tr><td>", "<\/tr><\/table> ") if /^\\sqrt\s*\{/;
  return ("$1<sub>$2</sub>", $') if /^\\(log|ln)_(\w+)/;
  return ("$1", $') if /^\\((log|ln)|(arc)?(sin|cos|tan))/;
  return math($', "<big><i>", "</i></big>") if /^\\mathcal\s*\{/;

  #special chars
  return ("&lt;", $') if /^</;
  return ("$gt;", $') if /^>/;
  return ("-&gt;", $') if /^\\mapsto/;
  return ("\\", $') if /^\\backslash/;
  return ($1, $') if /^\\(pi|theta)/;
  return ("&middot;" x 3, $') if /^\\cdots/;
  return ("&middot;", $') if /^\\cdot/;
  return ("\$", $') if /^\\\$/;

  #remove any other backslashes
  return ("", $') if /^\\/;


  #not recognized, so skip char
  return ($&, $') if /^./;
}

sub verbatim
{
  my $p;

  $_ = $_[0];

  /\\end\s*\{verbatim\*?\}/;

  $p = $`;
  $_ = $';

  $p =~ s/\\\\/\\/g;
  $p =~ s/&/&amp;/g;
  $p =~ s/</&lt;/g;
  $p =~ s/>/&gt;/g;

  return ("<pre>\n$p</pre>", $_);
}

sub table
{
  my ($l, $p, $r, $col, %alignalias, @align);

  $_ = $_[0];
  s/^\s+//;

  %alignalias = ("l", "left", "r", "right", "c", "center");

  @align = map $alignalias{$_}, split /\s+/, $1 if /^\{(\w(\s+\w)*)\}/;

  s/^[^\n]*\n//;

  $p = "";
  $r = "";
  $col = 0;

  until(/^\}/)
  {
    ($l, $_) = latex($_, "(&|\\\\\\\\|\\\\end\\s*\\{(table|tabular)\\*?)");

    $r .= "<td align=$align[$col]>$l</td>";
    $col++;

    if(/^\s*\n/)
    {
      $_ = $';
      $p .= "<tr>$r</tr>";
      $r = "";
      $col = 0;
    }
  }

  $p .= "<tr>$r</tr>" if $r ne "";

  /^\}/;
  return ("$_[1]$p$_[2]", $');
}

sub languagestrings
{
  my $s = pop;
  $s =~ s/$strings/$strings{$&}/g;
  return $s;
}

sub load
{
  open FILE,"langs.txt";
  my $b = $/;
  $/ = undef;
  $_ = <FILE>;
  $/ = $b;
  $_ = $& if /^\[$_[0]\][^\[\]]*/mi || /^\[English\][^\[\]]*/mi;
  #load language-dependent strings
  $strings{$1} = $2 while s/^(\S+)\s*=\s*(.+)$//mi;
  $strings = join "|",keys %strings;
  ${$_} = $strings{$_} foreach (keys %strings);
}

1;
