#!/usr/bin/perl

require 'lib.pl';

if(scalar(@ARGV) < 3)
{
  print <<EOF;

Latex-to-HTML converter
by Jonah Cohen <jonah\@ibiblio.org>
http://www.ibiblio.org/obp/ltoh/

Usage:
  $0 filename source_dir dest_dir [language]

Converts from source_dir/filename.tex to dest_dir/filename.htm

EOF
  exit(0);
}

load($ARGV[3]);

open INPUT, "$ARGV[1]/$ARGV[0].tex" || die "Can't read from $ARGV[1]/$ARGV[0].htm\n";
$b = $/;
$/ = undef;
$lines = <INPUT>;
$/ = $b;
close INPUT;

$chapnum = $ARGV[0];
$chapnum =~ s/[^\d]//g;
$chap = $chapnum;
$chap =~ s/^0//;

open OUTPUT, ">$ARGV[2]/$ARGV[0].htm" || die "Can't write to $ARGV[2]/$ARGV[0].htm\n";

push @files,"index";
open TOP, "$ARGV[1]/top.tex" || die "Can't read from $ARGV[1]/top.tex\n";
$_ = <TOP>;
close TOP;
push @files, $1 while s/^[^%\n]*\\include\s*\{([^\}]*)\}//m;
push @files, "dex";

open LABELS, ".labels_lst";
foreach (split /\n/,<LABELS>)
{
  /(.*)\t(.*)\t(.*)/;
  $labels{$1} = $2;
  $labellinks{$1} = $3;
}
close LABELS;

open INDEX, ".index_lst";
foreach (split /\n/,<INDEX>)
{
  /(.*)\t(.*)/;
  $index{$1} = $2;
}
close INDEX;

open HEADER, "html_header.html" || die "Can't read from html_header.html\n";
$header = languagestrings(<HEADER>);
close HEADER;

open HEADERFOOTER, "html_header_footer.html" || die "Can't read html header\n";
$headerfooter = languagestrings(<HEADERFOOTER>);
close HEADERFOOTER;

print "Converting $ARGV[0].tex to $ARGV[0].htm\n";

#find previous and next files
for($i=0; $i<scalar(@files); $i++)
{
  if($ARGV[0] =~ $files[$i])
  {
    $previous = $files[$i-1];
    $next = $files[$i+1];
    last;
  }
}

#find title
$chapname = $1 if $lines =~ /^[^%]*\\chapter\s*\{([^\}]*)\}/m;

if($ARGV[0] =~ /ch/)
{
  $title = "$Chapter $chap: $chapname";
  $chapter = "\n<h2>$Chapter $chap</h2>\n";
  $ischapter = 1;
}
elsif($ARGV[0] =~ /ap/)
{
  $title = "$Appendix " . pack('c',64+$chap) . ": $chapname";
  $chapter = "\n<h2>$Appendix " . pack('c',64+$chap) . "</h2>\n";
}
else
{
  $title = $chapname;
  $chapter = "";
}

$header =~ s/\$title/$title/;
$headerfooter =~ s/\$previous/$previous/gm;
$headerfooter =~ s/\$next/$next/gm;

$body = join "", latex($lines);

print OUTPUT <<EOF;
$header
$headerfooter
$chapter
$body
<p>
$headerfooter
<script language="Javascript">
<!--
function fn(str)
{
  var w = window.open("","newwin","height=200,width=200");
  w.document.write("<html><head><title>Note</title>$style<\\/head><body bgcolor=white>" + str + "<p><a href=\\"javascript:self.close();\\"><small><font color=blue><b>Close window</b></small></font></a></p></body></html>");
}
//-->
</script>
</body>
</html>
EOF

close OUTPUT;