#!/usr/bin/perl

require 'lib.pl';

if(scalar(@ARGV) < 1)
{
  print <<EOF;

Latex Parser
by Jonah Cohen <jonah\@ibiblio.org>
http://www.ibiblio.org/obp/ltoh/

Usage:
  $0 file source_dir/ [language]

Parses info from source_dir/file.tex

EOF
  exit(0);
}
load($ARGV[2]);

open(INPUT, "$ARGV[1]/$ARGV[0].tex") || die("Can't read from $ARGV[1]/$ARGV[0].tex\n");
@lines = split /\n/,<INPUT>;
close(INPUT);

$chap = $ARGV[0];
$chap =~ s/^.*?(\d+)$/$1/;
$chapnum = $chap + 0;

open(LABELS, ".labels_lst");
foreach (split /\n/,<LABELS>)
{
  /(.*)\t(.*)\t(.*)/;
  $labels{$1} = $2;
  $labellinks{$1} = $3;
}
close(LABELS);

open(LABELS, ">.labels_lst") || die("Can't write to .labels_lst\n");

open(INDEX, ".index_lst");
foreach (split /\n/,<INDEX>)
{
  /(.*)\t(.*)/;
  $index{$1} = $2;
}
close(INDEX);

open(INDEX, ">.index_lst")  || die("Can't write to .index_lst.\n");

print "Parsing $ARGV[0].tex\n";

foreach (@lines)
{
  $sec += /\\section\s*\{/;

  #look for index tags
  while(/\\index\s*\{/)
  {
  	($i, $_) = latex($');
#  	print "$i\n";
#    $i =~ s/<\/?p>//g;

    $indexlink = "$ARGV[0].htm";
    if($ARGV[0] =~ /ch/)
    {
      $index = $chapnum;
      if($sec)
      {
        $index .= ".$sec";
        $indexlink .= "#$sec";
      }
    }
    else
    {
      $index = "$Appendix " . pack('c',64+$chap);
    }
    if($i ne "")
    {
      %indices = ();
      foreach (split /,/,$index{$i})
      {
        /-/;
        $indices{$`} = $';
      }
      $indices{$index} = $indexlink;
      $index{$i} = join ",",map "$_-$indices{$_}",sort {sec($a) <=> sec($b)} keys %indices;
    }
  }

  #look for labels
  while(s/\\label\s*\{([^\}]*)\}//)
  {
    $l = $1;
    $l =~ s/^\s*//;
    $l =~ s/\s*$//;
    if($ARGV[0] =~ /ch/)
    {
      $labels{$l} = 0+$chap;
      $labellinks{$l} = "chap$chap.htm";
      if($sec)
      {
        $labels{$l} .= ".$sec";
        $labellinks{$l} .= "#$sec";
      }
    }
    else
    {
      $labels{$l} = pack('c',64+$chap);
      $labellinks{$l} = "app$chap.htm";
    }
  }
}

foreach (keys %labels)
{
  print LABELS "$_\t$labels{$_}\t$labellinks{$_}\n";
}
close(LABELS);

foreach (keys %index)
{
  print INDEX "$_\t$index{$_}\n";
}
close(INDEX);

sub sec
{
  $_ = pop;
  return 100 + unpack('c', $_) unless /\./;
  s/\.(\d)$/.0$1/;
  return $_;
}
