#!/usr/bin/perl

require 'lib.pl';

if(scalar(@ARGV) < 2)
{
  print <<EOF;

Table of Contents Generator
by Jonah Cohen <jonah\@ibiblio.org>
http://www.ibiblio.org/obp/ltoh/

Usage:
  $0 source_dir dest_dir [language]

Parses info from file.tex

EOF
  exit(0);
}

load($ARGV[2]);

open(LANGS, "ls *.lang|");
$langs = <LANGS>;
close(LANGS);

$langs =~ s/\.lang//g;
$langs =~ s/[^\w\n]/\\$&/g;
$langs =~ s/\n/|/gm;
chop $langs;

open(TOP, "$ARGV[0]/top.tex") || die("Can't open $ARGV[0]/top.tex\n");
$_ = <TOP>;
close(TOP);

$top = $_;

$title = $1 if /^[^%\n]*\\title\s*\{([^\}]*)\}/m;
$author = $1 if /^[^%\n]*\\author\s*\{([^\}]*)\}/m;
$lang = ucfirst lc $& if /$langs/mi;

$title =~ s/(\w+)/ucfirst $1/ge;
$title =~ s/\\\w*\s*//g;

open(OUTPUT, ">$ARGV[1]/index.htm") || die("Can't write to $ARGV[1]/index.htm.\n");

print "Creating Table of Contents\n";

print OUTPUT <<EOF;
<html>

<head>
  <title>$title</title>
  <style type="text/css">

body {background-color: #fffacd; text-align: center}
a {text-decoration: none; color: #a0522d; font-size: 11pt;
font-weight: bold; font-family: Verdana}
a:hover {text-decoration: underline}
a:active {text-decoration: underline}
a.h2 {font-size: 15pt; font-family: Times}
a.h3 {font-size: 12pt; font-family: Times}
h1 {font-size: 22pt; font-family: Times}
p.h2 {font-size: 15pt; font-family: Times}
table {border: 0; margin-left: auto; margin-right: auto}
td {background-color: #b0e0e6}
th {font-size: 12pt; font-family: Times}

</style>
</head>

<body>
  <h1>$title</h1>
  <h2>$lang Version</h2>
  <p class=h2>by $author</p>

<table cellspacing=6 cellpadding=7 align=center>
  <tr>
    <th>$TOC</th>
  </tr>
EOF

while($top =~ /^[^%\n]*\\include\s*\{([^\}]*)\}/m)
{
  $top = $';
  $file = $1;
  $n = $1;
  $n =~ s/[^\d]//g;
  $n =~ s/^0//;

  #get title
  open(FILE, "$ARGV[0]/$file.tex") || next;
  $_ = <FILE>;
  close(FILE);
  ($title, $_) = latex($') if /\\chapter\s*\{/;

  if($file =~ /ch/)
  {
    $title = "$Chapter $n: $title";
  }
  elsif($file =~ /ap/)
  {
    $title = "$Appendix " . pack('c',64+$n) . ": $title";
  }

  print OUTPUT <<EOF;
  <tr>
    <td><a href="$file.htm">$title</a></td>
  </tr>
EOF
}

print OUTPUT <<EOF;
  <tr>
    <td><a href="dex.htm">$Index</a></td>
  </tr>
</table>

</body>
</html>
EOF

close(OUTPUT);
